==> jmb.address-0.4.tar.gz: jmb.address-0.4/jmb/address/models.py <==
# -*- coding: utf-8 -*-
import math
from jmb.address.managers import NearbyManager

import requests, time
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jmb.l10n.models import Country
from jmb.address.utils import get_default_country, get_distance, get_google_geocoding


#######################################
# INDIRIZZO FINALITA                  #
#######################################
class AddressScope(DateModel, UserModel, StatusModel, OrderedModel):
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    is_invoice_address = models.BooleanField(
        default=False,
        verbose_name=_('Invoice Address'),
        help_text=_("If this address scope is an invoice address")
    )
    is_shipping_address = models.BooleanField(
        default=False,
        verbose_name=_('Shipping Address'),
        help_text=_("If this address scope is a shipping address")
    )

    class Meta:
        verbose_name = _("address scope")
        verbose_name_plural = _("address scopes")

    def __unicode__(self):
        return unicode(self.name)

    def _get_default_country(self):
        return Country.objects.get(iso2_code='IT')


###############################
# INDIRIZZO                   #
###############################
class Address(DateModel, UserModel, StatusModel, OrderedModel):
    address_scopes = models.ManyToManyField(
        AddressScope,
        db_index=True,
        verbose_name=_("address scopes"),
    )
    address_type = models.IntegerField(
        default=7,
        choices=settings.ADDRESS_ADDRESS_TYPE_CHOICES,
        db_index=True,
        verbose_name=_("address type"),
        blank=True, null=True,
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    number = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("number"),
        db_column="anumber"
    )
    crossed = models.CharField(
        max_length=5,
        blank=True, null=True,
        verbose_name=_("crossed"),
    )
    scale = models.CharField(
        max_length=5,
        blank=True, null=True,
        verbose_name=_("scale"),
    )
    floor = models.CharField(
        max_length=3,
        blank=True, null=True,
        verbose_name=_("floor"),
    )
    zip_code = models.CharField(
        max_length=20,
        db_index=True,
        verbose_name=_("zip code"),
        blank=True, null=True,
    )
    locality = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("locality"),
        blank=True, null=True,
    )
    province = models.CharField(
        max_length=3,
        db_index=True,
        verbose_name=_("province"),
        blank=True, null=True,
    )
    country = models.ForeignKey(
        Country,
        db_index=True,
        default=get_default_country,
        verbose_name=_("country"),
    )
    post_office = models.CharField(
        max_length=10,
        blank=True, null=True,
        verbose_name=_("post office"),
    )

    def get_scopes(self):
        scopes = self.address_scopes.all()
        names = [scope.name for scope in scopes]
        return ", ".join(names)

    get_scopes.short_description = _("scopes")

    def get_organizations(self):
        organizations = self.organization_set.all()
        names = [organization.name for organization in organizations]
        return ", ".join(names)

    get_organizations.short_description = _("organizations")

    def get_gmaps_address(self):
        return "%(address_type)s %(name)s, %(number)s, %(cap)s, %(locality)s" % {
            'address_type': self.get_address_type_display() or '',
            'name': self.name,
            'number': self.number,
            'locality': self.locality,
            'cap': self.zip_code or ''}

    gmaps_address = property(get_gmaps_address)

    class Meta:
        abstract = True
        verbose_name = _("address")
        verbose_name_plural = _("addresses")

    def __unicode__(self):
        return u"%(address_type)s %(name)s %(number)s - %(locality)s (%(province)s) - %(country)s" % {
            'address_type': self.get_address_type_display(),
            'name': self.name, 'number': self.number or '',
            'locality': self.locality, 'province': self.province,
            'country': self.country
        }


class AddressCoordinates(models.Model):
    """
    This abstract model is used when you want to add latitude and longitude coordinates to a concrete model.
    You must define in the inherited model an attribute called 'geocoding_fields' which is used to
    get and save coordinates via google map API (v3)
    """
    lat = models.DecimalField(
        verbose_name=_("Latitude"),
        blank=True, null=True,
        max_digits=20, decimal_places=17,
    )
    lng = models.DecimalField(
        verbose_name=_("Longitude"),
        blank=True, null=True,
        max_digits=20, decimal_places=17,
    )
    geocoding_fields = []

    """
    NearbyManager gives access to the method .nearby(lat, lng, proximity), which returns all objects within the range in km (proximity)
    from the point(lat, lng)
    ie. AddressCoordinates.objects.nearby(lat, lng, proximity)
    """
    objects = NearbyManager()

    class Meta:
        abstract = True
        verbose_name = _("address coordinates")
        verbose_name_plural = _("addresses coordinates")

    def __unicode__(self):
        return u"lat: {0}, lng: {1}".format(self.lat, self.lng)

    @property
    def address_search_querystring(self):
        return " ".join(
            [getattr(self, field).strip() for field in self.geocoding_fields if
             isinstance(getattr(self, field), (str, unicode)) and getattr(self, field).strip()]
        ).replace(" ", "+")

    def get_google_geocoding(self):
        return get_google_geocoding(self.address_search_querystring)

    def save(self, *args, **kwargs):
        if self.geocoding_fields and not self.id:
            coordinates = self.get_google_geocoding()
            if coordinates['lat'] and coordinates['lng']:
                self.lat = coordinates['lat']
                self.lng = coordinates['lng']
        super(AddressCoordinates, self).save(*args, **kwargs)

    def google_map(self):
        context = dict()
        return render_to_string('admin/address/addresscoordinates/google_map.html', context)

    google_map.allow_tags = True
    google_map.short_description = _("Map")

    def get_distance(self, latitude, longitude, radius=6371):
        """
        Uses the Haversine formula, resulting in an error of 0.5%
        By default, returns distance in km
        """
        return get_distance(longitude, latitude, self.lng, self.lat, radius)

==> jmb.address-0.5.tar.gz: jmb.address-0.5/jmb/address/models.py <==
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from jmb.address.managers import NearbyManager

from django.utils.encoding import python_2_unicode_compatible
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jmb.l10n.models import Country
from jmb.address.utils import get_default_country, get_distance, get_google_geocoding


# INDIRIZZO FINALITA
@python_2_unicode_compatible
class AddressScope(DateModel, UserModel, StatusModel, OrderedModel):
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    is_invoice_address = models.BooleanField(
        default=False,
        verbose_name=_('Invoice Address'),
        help_text=_("If this address scope is an invoice address")
    )
    is_shipping_address = models.BooleanField(
        default=False,
        verbose_name=_('Shipping Address'),
        help_text=_("If this address scope is a shipping address")
    )

    class Meta:
        verbose_name = _("address scope")
        verbose_name_plural = _("address scopes")

    def __str__(self):
        return str(self.name)

    def _get_default_country(self):
        return Country.objects.get(iso2_code='IT')


# INDIRIZZO
@python_2_unicode_compatible
class Address(DateModel, UserModel, StatusModel, OrderedModel):
    address_scopes = models.ManyToManyField(
        'AddressScope',
        db_index=True,
        verbose_name=_("address scopes"),
    )
    address_type = models.IntegerField(
        default=7,
        choices=settings.ADDRESS_ADDRESS_TYPE_CHOICES,
        db_index=True,
        verbose_name=_("address type"),
        blank=True, null=True,
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    number = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("number"),
        db_column="anumber"
    )
    crossed = models.CharField(
        max_length=5,
        blank=True, null=True,
        verbose_name=_("crossed"),
    )
    scale = models.CharField(
        max_length=5,
        blank=True, null=True,
        verbose_name=_("scale"),
    )
    floor = models.CharField(
        max_length=3,
        blank=True, null=True,
        verbose_name=_("floor"),
    )
    zip_code = models.CharField(
        max_length=20,
        db_index=True,
        verbose_name=_("zip code"),
        blank=True, null=True,
    )
    locality = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("locality"),
        blank=True, null=True,
    )
    province = models.CharField(
        max_length=3,
        db_index=True,
        verbose_name=_("province"),
        blank=True, null=True,
    )
    country = models.ForeignKey(
        Country,
        db_index=True,
        default=get_default_country,
        verbose_name=_("country"),
    )
    post_office = models.CharField(
        max_length=10,
        blank=True, null=True,
        verbose_name=_("post office"),
    )

    def get_scopes(self):
        scopes = self.address_scopes.all()
        names = [scope.name for scope in scopes]
        return ", ".join(names)

    get_scopes.short_description = _("scopes")

    def get_organizations(self):
        organizations = self.organization_set.all()
        names = [organization.name for organization in organizations]
        return ", ".join(names)

    get_organizations.short_description = _("organizations")

    def get_gmaps_address(self):
        return "%(address_type)s %(name)s, %(number)s, %(cap)s, %(locality)s" % {
            'address_type': self.get_address_type_display() or '',
            'name': self.name,
            'number': self.number,
            'locality': self.locality,
            'cap': self.zip_code or ''}

    gmaps_address = property(get_gmaps_address)

    class Meta:
        abstract = True
        verbose_name = _("address")
        verbose_name_plural = _("addresses")

    def __str__(self):
        return "%(address_type)s %(name)s %(number)s - %(locality)s (%(province)s) - %(country)s" % {
            'address_type': self.get_address_type_display(),
            'name': self.name, 'number': self.number or '',
            'locality': self.locality, 'province': self.province,
            'country': self.country
        }


@python_2_unicode_compatible
class AddressCoordinates(models.Model):

    """
    This abstract model is used when you want to add latitude and longitude coordinates to a concrete model.
    You must define in the inherited model an attribute called 'geocoding_fields' which is used to
    get and save coordinates via google map API (v3)
    """
    lat = models.DecimalField(
        verbose_name=_("Latitude"),
        blank=True, null=True,
        max_digits=20, decimal_places=17,
    )
    lng = models.DecimalField(
        verbose_name=_("Longitude"),
        blank=True, null=True,
        max_digits=20, decimal_places=17,
    )
    geocoding_fields = []

    """
    NearbyManager gives access to the method .nearby(lat, lng, proximity), which returns all objects within the range in km (proximity)
    from the point(lat, lng)
    ie. AddressCoordinates.objects.nearby(lat, lng, proximity)
    """
    objects = NearbyManager()

    class Meta:
        abstract = True
        verbose_name = _("address coordinates")
        verbose_name_plural = _("addresses coordinates")

    def __str__(self):
        return "lat: {0}, lng: {1}".format(self.lat, self.lng)

    @property
    def address_search_querystring(self):
        return " ".join(
            [getattr(self, field).strip() for field in self.geocoding_fields if
             isinstance(getattr(self, field), str) and getattr(self, field).strip()]
        ).replace(" ", "+")

    def get_google_geocoding(self):
        return get_google_geocoding(self.address_search_querystring)

    def save(self, *args, **kwargs):
        if self.geocoding_fields and not self.id:
            coordinates = self.get_google_geocoding()
            if coordinates['lat'] and coordinates['lng']:
                self.lat = coordinates['lat']
                self.lng = coordinates['lng']
        super(AddressCoordinates, self).save(*args, **kwargs)

    def google_map(self):
        context = dict()
        return render_to_string('admin/address/addresscoordinates/google_map.html', context)

    google_map.allow_tags = True
    google_map.short_description = _("Map")

    def get_distance(self, latitude, longitude, radius=6371):
        """
        Uses the Haversine formula, resulting in an error of 0.5%
        By default, returns distance in km
        """
        return get_distance(longitude, latitude, self.lng, self.lat, radius)
